# PI Problem Database

This project aims to process Schittkowski's collection of dynamical systems for testing data fitting software.
See the [documentation](./doc/ds_test_problems.pdf) for more information.

**Remark:** Some of the original examples and models were modified:

* `PHA_DYN1`, `PHA_DYN3`, `PHA_DYN4`, `PHA_DYN5`, `PHA_DYN6`, `PHA_DYN7`, `PHA_REAC`: datasets contained measurement rows with negative time points, where all measurement values were weighted with zero. These lines were removed.
* `GLUCOSE2`: was classified as `differential algebraic equation` but should be `ordinary differential equation`; The measured states were not sorted, and the last column was weighted with zero anyway. This column was removed, and consequently, the ordering was fixed.
* `PHARM_AP`: was classified as `ordinary differential equation` but should be `steady state equation`.
* `SS_TUBE`: was classified as `ordinary differential equation` but should be `partial differential equation`.
* `ENTERO`: typo in the initial value function
* `GAS_OIL`: typo in the initial value function
* `PHARMA`: typo in the initial value function
* `BRUSSEL1`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and consequently, the ordering was fixed.
* `CSTR_JY0`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and consequently, the ordering was fixed.
* `OREGO`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and consequently, the ordering was fixed.
* `NTA1`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and the fitting functions were reordered.
* `NTA2`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and the fitting functions were reordered.
* `ASS_CV1`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and the fitting functions were reordered.
* `TRANSUS1`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and consequently, the ordering was fixed.
* `TRANSUS2`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and consequently, the ordering was fixed.
* `SEIR`: The measured states were not sorted, and many columns were just weighted with zero. These columns were removed, and consequently, the ordering was fixed.
* `CONSREA`: Datasets contained duplicate time points. The corresponding measurements could be merged because one entry was always weighted with zero.
* `FOLDING1`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `FOLDING2`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `FOLDING4`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `COAL2`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `COAL5`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `HOMPOLY`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `FRACTAK`: Datasets contained duplicate time points. The corresponding measurements could be merged because one entry was always weighted with zero; The last measurement entry, which caused the fitting function to be non-standard was weighted with zero and thus removed.
* `MOTION`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `NC_RHS`: Datasets contained duplicate first time point. The second measurement row however looked like it was accidentally added and thus removed.
* `OLIGO`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `ON_OFF3`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible; The last measurement entry, which caused the fitting function to be non-standard was weighted with zero and thus removed.
* `ON_OFF5`: The last measurement entry, which caused the fitting function to be non-standard was weighted with zero and thus removed.
* `ON_OFF6`: The last measurement entry, which caused the fitting function to be non-standard was weighted with zero and thus removed.
* `ON_OFF7`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible; The last measurement entry, which caused the fitting function to be non-standard was weighted with zero and thus removed.
* `OXENZ`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `POPUL`: Datasets contained duplicate time points. The corresponding measurements were merged into one data point. As there are many data points, the effect should be negligible.
* `SPRING`: Datasets contained a duplicate row. It was removed.
* `STIFF2`: Datasets contained duplicate rows. They were removed.
* `ASS_CV4`: Remove the last concentration block because it is weighted with zero.
* `ASS_KIN3`: Remove a concentration block because it is weighted with zero.
* For the following datasets columns where all weights were zero are removed: `BATEX`, `CAR_CTR1`, `CAR_CTR2`, `COAL1`, `CONVER`, `DIFDIST`, `DRYER`, `PHA_DYN4`, `MEMSEP`, `SBR_COOL`, `LKIN_NUM`, `PHA_DYN7`, `CASC_IMP`, `EQMULTI`, `ASS_CV6`, `RE_ENTRY`, `ISOTOP1`, `REXT`, `BATCH_CT`, `ORBIT`, `DRUG_SCH`, `LKIN_T`, `OSCILL`, `PHA_DYN1`, `TOPTCSTR`, `DEACTENZ`, `LANDING`, `PHA_DYN3`, `RT_PULSE`, `ROBOT_2`, `INHIB`, `DRY_FRI1`, `COLCON`, `OSC_REAC`, `MOT_TSP`, `OC_EX4`, `FERMPROC`, `PYROLYS`, `DRY_FRI3`, `PHA_DYN5`, `OXDYN`, `POLYBU`, `VDPOL_0`, `HIRUDIN`, `FBR`, `TAB_DIS3`, `TAB_DIS4`, `TAB_DIS5`, `TAB_DIS6`, `TAB_DIS7`, `OXIDAT`, `CSTR_CTL`, `SOOT`, `DCIMMIGR`, `NL_CSTR`, `BIO_REAC`, `LASER`, `ON_OFF4`, `COMPASM`, `STAGED`, `DISPLMNT`, `COLLISIO`, `ASS_CV7`, `ANAEMEAS`, `CSTR_CTR`, `SCP`, `SAT_EXP`, `EQBACK`, `ORB_MOTN`, `CATALYST`, `GLUCOSE1`, `CARGO`, `TWOSTAGE`, `TANKHYD`, `PENICILL`, `REACMECH`, `PROTEIN`, `BIOREAC`, `FED10`, `NC_RHS`, `NITRIF`, `CONTUN`, `DIODE`, `ANHYD`, `TWOONE`, `DRY_FRI2`, `GASLIQ1`, `GASLIQ2`, `RES_TIME`, `CS_REAC`, `MEMINH`, `PHA_REAC`, `CAV_BUBB`, `PLUG_FLO`, `RABBIT`, `PECAN`, `OXENZ`, `HIGH_ORD`, `HOLDUP`, `NL_ODE`, `GYROS`, `SPBEDRTD`, `CONF_ALT`, `DUCT`, `PENDU_I0`, `OIL`, `ISO_BAT`, `FISH_POP`, `2CSTR`, `CRANE`, `BIO_MOD`, `LISA`, `PHA_DYN2`, `PHA_DYN6`, `GREASE`, `STABCSTR`

The original datasets and models are checked into this repository as `*.DAT_original` and `*.fun_original`.

This project defines CMake targets `PI-Problem-Database::MODEL_NAME`, where the valid values of `MODEL_NAME` are defined in the CMake Cache variable `PI-Problem-Database_Models`.
There are several options to use the library in your projects:

## 1. Submodule

You could simply add this repository as a submodule and use CMake's `add_subdirectory` to include it in your project. However, you might want to disable the installation option:

```CMake
set(Install_PI-Problem-Database
    OFF
    CACHE BOOL "Disable installation of PI-Problem-Database." FORCE)
add_subdirectory(PI-Problem-Database)
```

## 2. Fetch Content

It is also possible to add this library with CMake's Fetch Content command. Again, disabling the installation option is recommended:

```CMake
include(FetchContent)

FetchContent_Declare(
  PI-Problem-Database
  PREFIX "${CMAKE_BINARY_DIR}/PI-Problem-Database"
  GIT_REPOSITORY
    "git@gitlab.informatik.uni-bremen.de:mwiesner/pi-problem-database.git"
  GIT_TAG "origin/main")

set(Install_PI-Problem-Database
    OFF
    CACHE BOOL "Disable installation of PI-Problem-Database." FORCE)

FetchContent_MakeAvailable(PI-Problem-Database)
```

## 3. find_package

You could also install this library locally on your machine by building the `install` target (you might need `sudo` rights). It allows you to find the library in other CMake projects on your machine using the `find_package` command:

```CMake
find_package(PI-Problem-Database)
```

## Developer board

### Convert PCOMP to Fortran Code

To convert the PCOMP models to Fortran code, use the program `to_fortran`.
Compile it with the command

```Bash
gfortran -o to_fortran f90getopt.F90 PCOMP/* to_fortran.f90
```

and run

```Bash
./to_fortran -p PCOMP_MODEL_FILE -f OUTPUT_FORTRAN_FILE
```

Besides the Fortran code, the program also produces a log file, `log.txt`.

### Convert Dataset to C++ code

To convert the dataset to C++ Code, use the program `scripts/convert_dataset_to_dynamic_model_infos.py`:

```Bash
python3 ./scripts/convert_dataset_to_dynamic_model_infos.py DATASET_FILE TEMPLATE_HPP_FILE OUTPUT_HPP_FILE
```

### Read measurements and weights from Dataset and write to CSV files

To convert the datasets to C++ Code, use the program `scripts/convert_datasets_to_csv.py`:

```Bash
python3 ./scripts/convert_datasets_to_csv.py DATASET_DIRECTORY --outputFolder OUTPUT_DIRECTORY
```

### Read indices of model parameters in ODE from the model files and write to JSON

To read the indices of model parameters in ODE from the PCOMP file and write them to a JSON file, use the program `scripts/get_indices_of_model_parameters_in_ode.py`:

```Bash
python3 ./scripts/get_indices_of_model_parameters_in_ode.py DATASET_DIRECTORY MODELS_DIRECTORY OUTPUT_FILE
```

### Read indices of measured states from the model files and write to JSON

To read the indices of measured states from the PCOMP file and write them to a JSON file, use the program `scripts/get_indices_of_measured_states.py`:

```Bash
python3 ./scripts/get_indices_of_measured_states.py DATASET_DIRECTORY MODELS_DIRECTORY OUTPUT_FILE
```

### Read initial state info from the model files and write to JSON

To read info regarding the initial states from the PCOMP file and write them to a JSON file, use the program `scripts/get_initial_state_info.py`:

```Bash
python3 ./scripts/get_initial_state_info.py DATASET_DIRECTORY MODELS_DIRECTORY OUTPUT_FILE
```

### Read constraint status from the model files and write to JSON

To read the constraint status (indicator whether the constraints are valid, i.e. they depend only on model parameters which are also part of the ODE) from the PCOMP file and write it to a JSON file, use the program `scripts/get_constraint_status.py`:

```Bash
python3 ./scripts/get_constraint_status.py DATASET_DIRECTORY MODELS_DIRECTORY OUTPUT_FILE
```
