import os
from collections import defaultdict

import numpy as np

from src.Dataset import Dataset, ModelType
from src.read_dataset import read_general_info_to_dataset, read_datasets_from_directory


datasetsDirectory = "datasets"

datasetsTypes = {}
for fileName in os.listdir(datasetsDirectory):
    fileName = os.path.join(datasetsDirectory, fileName)
    if os.path.isfile(fileName):
        with open(fileName, encoding='utf-8', errors='ignore') as file:
            dataset = Dataset()
            linesIt = iter(file.readlines())
            read_general_info_to_dataset(linesIt, dataset)

            datasetsTypes[dataset.name] = dataset.modelType


groupedDatasets = defaultdict(list)

for modelName, modelType in datasetsTypes.items():
    groupedDatasets[modelType].append(modelName)

for modelType, modelNames in groupedDatasets.items():
    print(f"{modelType}: {len(modelNames)}")

for modelType, modelNames in groupedDatasets.items():
    print(f"\n{modelType}:\n{sorted(groupedDatasets[modelType])}")


datasets = read_datasets_from_directory(datasetsDirectory)
modelsWithZeroWeightsCount = 0
modelsWithZeroWeightsColumn = []
for dataset in datasets:
    if not dataset.modelType == ModelType.OrdinaryDifferentialEquations:
        continue
    if not dataset.measurements:
        print(f"Dataset {dataset.name} has no measurements. Skip it.")
    
    weights = [np.array(m.weights) for m in dataset.measurements]
    zeroWeightCountPerColumn = [np.count_nonzero(w == 0, axis=0) for w in weights]
    zeroWeightCount = [np.sum(w) for w in zeroWeightCountPerColumn]

    if any([count > 0 for count in zeroWeightCount]):
        message = f"{dataset.name} has zero weights"
        if any([w.shape[0] in countPerCol for w, countPerCol in zip(weights, zeroWeightCountPerColumn)]):
            message += f" It seems like whole weight columns are zero"
            modelsWithZeroWeightsColumn.append(dataset.name)
        
        print(message)

        modelsWithZeroWeightsCount += 1

print(f"Models with zero weights: {modelsWithZeroWeightsCount}")
print(f"Models with zero weights column: {len(modelsWithZeroWeightsColumn)}")
print('\n'.join(modelsWithZeroWeightsColumn))