from argparse import ArgumentParser
import logging

from loggingUtility import set_log_level, add_log_level_to_argument_parser

from src.to_cpp import write_dataset_to_cpp_header
from src.read_dataset import read_dataset_from_file
from src.Dataset import ModelType


def main() -> int:
    parser = ArgumentParser(description='Transform dataset to C++ header')
    parser.add_argument('datasetPath', type=str, help='Path to dataset')
    parser.add_argument('templateFilePath', type=str, help='Path to template header file')
    parser.add_argument('outputFilePath', type=str, help='Path to output header file')
    add_log_level_to_argument_parser(parser)
    args = parser.parse_args()
    set_log_level(args.logLevel)

    dataset = read_dataset_from_file(args.datasetPath)

    if dataset.modelType != ModelType.OrdinaryDifferentialEquations:
        logging.error(f"Dataset {args.datasetPath} is not an ODE model. It can not be transformed to C++ code.")
        return 1

    write_dataset_to_cpp_header(dataset, args.templateFilePath, args.outputFilePath)
    return 0


if __name__ == "__main__":
    exit(main())
