import logging
from argparse import ArgumentParser


def set_log_level(logLevel: str) -> None:
    validLoglevels = {
        'critical': logging.CRITICAL,
        'error': logging.ERROR,
        'warning': logging.WARNING,
        'info': logging.INFO,
        'debug': logging.DEBUG,
        'off': 'off'
    }

    level = validLoglevels.get(logLevel.lower())

    if level is None:
        raise ValueError(
            f"logLevel level given: {logLevel}"
            f" -- must be one of: {' | '.join(validLoglevels.keys())}")

    elif level == 'off':
        logging.disable()

    else:
        logging.basicConfig(level=level, format='%(asctime)s [%(levelname)s] %(message)s')


def add_log_level_to_argument_parser(parser: ArgumentParser):
    parser.add_argument(
        "--logLevel",
        default="info",
        choices=['debug', 'info', 'warning', 'error', 'critical', 'off'],
        help=("Log level, default is 'info'.")
    )
