from argparse import ArgumentParser
from pathlib import Path
import logging
import re

from loggingUtility import set_log_level, add_log_level_to_argument_parser


def main() -> int:
    parser = ArgumentParser(description='Post process the auto-generated Fortran model')
    parser.add_argument('fortranModelPath', type=str, help='Path to Fortran model')
    add_log_level_to_argument_parser(parser)
    args = parser.parse_args()
    set_log_level(args.logLevel)

    fortranModelPath = Path(args.fortranModelPath)

    logging.info(f"Post-process model '{fortranModelPath}'")

    fileContent = fortranModelPath.read_text()

    searchString = "LOGICAL ACTIVE"
    replaceString = "LOGICAL*1 ACTIVE"

    logging.info(f"Replace '{searchString}' with '{replaceString}'")

    fileContent, numberOfAppearances = re.subn(searchString, replaceString, fileContent, flags=re.IGNORECASE)

    expectedNumberOfAppearances = 2
    if numberOfAppearances != expectedNumberOfAppearances:
        logging.error(
            f"Expected {expectedNumberOfAppearances} appearances of pattern {searchString}, found {numberOfAppearances}")
        return 1

    fortranModelPath.write_text(fileContent)

    return 0


if __name__ == "__main__":
    exit(main())
