#include "InvalidFunctionCallException.hpp"

#include <gsl/gsl_assert>

namespace pi_database::dynamic_model
{

namespace
{

auto to_error_type(const int errorCode) -> ErrorType
{
  constexpr auto minPcompErrorCode = 1;
  constexpr auto maxPcompErrorCode = 71;
  Expects(errorCode >= minPcompErrorCode);
  Expects(errorCode <= maxPcompErrorCode);

  switch (errorCode) {
  case 9:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DivisionByZero;
  case 47:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainError;
  case 51:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainErrorAtanh;
  case 52:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainErrorLog;
  case 53:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainErrorSqrt;
  case 54:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainErrorAsin;
  case 55:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainErrorAcos;
  case 56:// NOLINT [cppcoreguidelines-avoid-magic-numbers]
    return ErrorType::DomainErrorAcosh;
  default:
    return ErrorType::Other;
  }
}

auto to_message(const int errorCode) -> std::string
{
  switch (to_error_type(errorCode)) {
  case ErrorType::DivisionByZero:
    return "Division by zero";
  case ErrorType::DomainError:
    return "Domain error";
  case ErrorType::DomainErrorAtanh:
    return "Domain error: atanh";
  case ErrorType::DomainErrorLog:
    return "Domain error: log";
  case ErrorType::DomainErrorSqrt:
    return "Domain error: sqrt";
  case ErrorType::DomainErrorAsin:
    return "Domain error: asin";
  case ErrorType::DomainErrorAcos:
    return "Domain error: acos";
  case ErrorType::DomainErrorAcosh:
    return "Domain error: acosh";
  case ErrorType::Other:
    return "Other error with code " + std::to_string(errorCode);
  default:
    Ensures(false);
  }

  Ensures(false);
}

}// namespace

InvalidFunctionCallException::InvalidFunctionCallException(int errorCode) :
  std::runtime_error{ to_message(errorCode) },
  errorCode{ errorCode }
{
  // nothing
}

auto InvalidFunctionCallException::get_error_type() const -> ErrorType
{
  return to_error_type(errorCode);
}

}// namespace pi_database::dynamic_model
