program to_fortran
   use f90getopt
   implicit none
   character(*), parameter ::logFilePath = "./log.txt"
   character(100) :: pcompFilePath, fortranFilePath
   integer :: status

   ! define options
   type(option_s) :: opts(3)
   opts(1) = option_s("pcompFile", .true., "-p")
   opts(2) = option_s("fortranFile", .true., "-f")

   ! processing options
   if (command_argument_count() == 0) then
      print *, "ERROR: Program has options: -p x --pcompFile=x -f x --fortranFile=x"
      stop
   end if

   do
      select case (getopt("p:f:", opts))
      case (char(0)) ! When all options are processed
         exit
      case ("p")
         pcompFilePath = trim(optarg)
         print *, "option pcompFile/p=", pcompFilePath
      case ("f")
         fortranFilePath = trim(optarg)
         print *, "option fortranFile/f=", fortranFilePath
      end select
   end do

   status = convert_to_fortran(pcompFilePath, fortranFilePath, logFilePath)
   if (status > 0) then
      call symerr(0, status)
      print *, "Conversion failed"
      stop
   end if

contains
   integer function convert_to_fortran(pcompFilePath, fortranFilePath, logFilePath) result(status)
      character(*), intent(in) :: pcompFilePath, fortranFilePath, logFilePath

      integer, parameter :: dp = kind(0.0d0)
      integer, parameter :: workingArraySize = 15000
      real(kind=dp) :: realWorkingArray(workingArraySize)
      integer :: integerWokingArray(workingArraySize)
      integer :: realStorageSize, integerStorageSize
      integer :: lineNumberOfError
      integer :: mode = 2 ! 0=function, 1=function+gradient, 2=function+gradiant+hessian
      integer :: numberOfVariables, numberOfFunctions
      integer :: debugLevel = 3

      integer :: pcompFileUnit = 2, symFileUnit = 3, fortranFileUnit = 4, logFileUnit = 5

      status = 0

      open (pcompFileUnit, file=pcompFilePath)
      open (symFileUnit, file='tmp.sym')
      open (fortranFileUnit, file=fortranFilePath)
      open (logFileUnit, file=logFilePath)

      print *, "Convert ", trim(pcompFilePath), " to intermediate code"
      call syminp( &
         pcompFileUnit, &
         symFileUnit, &
         realWorkingArray, &
         workingArraySize, &
         integerWokingArray, &
         workingArraySize, &
         realStorageSize, &
         integerStorageSize, &
         status, &
         lineNumberOfError, &
         mode, &
         numberOfVariables, &
         numberOfFunctions, &
         logFileUnit, &
         debugLevel)

      if (status > 0) then
         print *, "Conversion failed"
         return
      end if

      close (symFileUnit)
      open (symFileUnit, file='tmp.sym')

      print *, "Prepare intermediate code for conversion to Fortran"
      call symprp( &
         symFileUnit, &
         realWorkingArray, &
         workingArraySize, &
         integerWokingArray, &
         workingArraySize, &
         realStorageSize, &
         integerStorageSize, &
         status, &
         mode, &
         numberOfVariables, &
         numberOfFunctions)

      if (status > 0) then
         print *, "Preparation failed"
         return
      end if

      print *, "Convert intermediate code to Fortran code ", trim(fortranFilePath)
      call symfor(fortranFileUnit, &
                  realWorkingArray, &
                  workingArraySize, &
                  integerWokingArray, &
                  workingArraySize, &
                  status)

      if (status > 0) then
         print *, "Conversion failed"
         return
      end if

      close (pcompFileUnit)
      close (symFileUnit, status="delete")
      close (logFileUnit)
      close (fortranFileUnit)
   end function convert_to_fortran
end program to_fortran

