#include "2BODY.hpp"

#include <cmath>

namespace test_model
{

std::vector<double> ode(
  const double /*time*/,
  const std::vector<double>& states,
  const double /*concentration*/,
  const std::vector<double>& modelParameters)
{
  const auto a = modelParameters[0];
  const auto b = modelParameters[1];

  const auto y1 = states[0];
  const auto y2 = states[1];
  const auto y3 = states[2];
  const auto y4 = states[3];

  const auto r = y2 * y2 + y4 * y4;
  const auto rCube = r * r * r;

  return { y2, -a * y1 / rCube, y4, -b * y3 / rCube };
}

std::vector<double> getInitialValues(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const double concentration,
  const std::vector<double>& /*modelParameters*/)
{
  return { 1.0 - concentration, 0.0, 0.0, std::sqrt((1.0 + concentration) / (1.0 - concentration)) };
}

std::vector<double> fittingFunction(
  const double /*time*/,
  const std::vector<double>& states,
  const double /*concentration*/,
  const std::vector<double>& /*modelParameters*/)
{
  const auto y2 = states[1];
  const auto y4 = states[3];
  return { y2, y4 };
}

std::vector<double> constraints(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const double /*concentration*/,
  const std::vector<double>& /*modelParameters*/)
{
  return {};
}

}// namespace test_model
