#include "CASTOR.hpp"

#include <cmath>

namespace test_model
{

std::vector<double> ode(
  const double /*time*/,
  const std::vector<double>& states,
  const double concentration,
  const std::vector<double>& modelParameters)
{
  constexpr auto M = 227.0;
  constexpr auto H = 1050.0;

  const auto CA0 = modelParameters[0];
  const auto CP = modelParameters[1];

  const auto X = states[0];
  const auto TR = states[1];

  const auto Q = concentration;

  const auto M1 = M * CA0;
  auto K = 0;
  if (TR != 0) {
    K = (1.0 / 60) * std::exp(35.2 - 22450.0 / TR);
  }

  const auto dX = K * (1 - X);
  const auto dTR = (-M1 * H * K * (1 - X) + Q) / (M * CP);

  return { dX, dTR };
}

std::vector<double> getInitialValues(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const double /*concentration*/,
  const std::vector<double>& /*modelParameters*/)
{
  constexpr auto T0 = 631;
  return { 0.0, T0 };
}

std::vector<double> fittingFunction(
  const double /*time*/,
  const std::vector<double>& states,
  const double /*concentration*/,
  const std::vector<double>& /*modelParameters*/)
{
  return { states[0], states[1] };
}

std::vector<double> constraints(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const double /*concentration*/,
  const std::vector<double>& /*modelParameters*/)
{
  return {};
}

}// namespace test_model
