#include "HAMILTON.hpp"

#include <cmath>

namespace test_model
{

std::vector<double> ode(
  const double /*time*/,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  constexpr auto a = 0.1;
  constexpr auto b = 0.1;

  const auto x = states[0];
  const auto xa = states[1];
  const auto q = states[2];
  const auto p1 = states[3];
  const auto p2 = states[4];

  const auto x_t = -a * x - b * x * x - p1 / q;
  const auto xa_t = -x - a * xa - 2 * b * x * xa;
  const auto q_t = 0;
  const auto p1_t = (a + 2 * b * x) * p1 + (1 + 2 * b * xa) * p2;
  const auto p2_t = xa + (a + 2 * b * x) * p2;
  const auto p3_t = -p1 * p1 / (2 * q * q);

  return { x_t, xa_t, q_t, p1_t, p2_t, p3_t };
}

std::vector<double> getInitialValues(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const std::vector<double>& modelParameters)
{
  constexpr auto x0 = 0.1;

  const auto q0 = modelParameters[0];
  const auto p10 = modelParameters[1];
  const auto p20 = modelParameters[2];

  const auto x_0 = x0;
  const auto xa_0 = 0.0;
  const auto q_0 = q0;
  const auto p1_0 = p10;
  const auto p2_0 = p20;
  const auto p3_0 = 0.0;

  return { x_0, xa_0, q_0, p1_0, p2_0, p3_0 };
}

std::vector<double> fittingFunction(
  const double /*time*/,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  const auto x = states[0];
  const auto xa = states[1];
  const auto q = states[2];
  const auto p1 = states[3];
  const auto p2 = states[4];
  const auto p3 = states[5];

  const auto E_fit = -p3 * 2;
  const auto x_fit = x;
  const auto xa_fit = xa;
  const auto q_fit = q;
  const auto p1_fit = p1;
  const auto p2_fit = p2;
  const auto p3_fit = p3;

  return { E_fit, x_fit, xa_fit, q_fit, p1_fit, p2_fit, p3_fit };
}

std::vector<double> constraints(
  double /*time*/,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  const auto p1 = states[3];
  const auto p2 = states[4];

  return { p1, p2 };
}

}// namespace test_model
