#include "HOLDUP.hpp"

#include <cmath>

namespace test_model
{

std::vector<double> ode(
  const double time,
  const std::vector<double>& states,
  const std::vector<double>& modelParameters)
{
  constexpr auto L0 = 13.05;
  constexpr auto M = 0.7605;
  constexpr auto N = 2.805E-2;

  const auto H1 = states[0];
  const auto H2 = states[1];
  const auto H3 = states[2];
  const auto H4 = states[3];
  const auto H5 = states[4];
  const auto H6 = states[5];
  const auto H7 = states[6];

  const auto Z0 = modelParameters[0];
  const auto Z = modelParameters[1];
  const auto Z8 = modelParameters[2];

  const auto G0 = time < 4 ? 10.5 : 12.0;

  const auto G1 = std::pow(H1, M) / N;
  const auto G2 = std::pow(H2, M) / N;
  const auto G3 = std::pow(H3, M) / N;
  const auto G4 = std::pow(H4, M) / N;
  const auto G5 = std::pow(H5, M) / N;
  const auto G6 = std::pow(H6, M) / N;
  const auto G7 = std::pow(H7, M) / N;
  const auto G8 = G0;

  const auto L1 = L0 + G2 - G1;
  const auto L2 = L1 + G3 - G2;
  const auto L3 = L2 + G4 - G3;
  const auto L4 = L3 + G5 - G4;
  const auto L5 = L4 + G6 - G5;
  const auto L6 = L5 + G7 - G6;
  // const auto L7 = L6 + G8 - G7;

  const auto H1_t = (G2 - G1) / Z0;
  const auto H2_t = (G3 - G2) / Z;
  const auto H3_t = (G4 - G3) / Z;
  const auto H4_t = (G5 - G4) / Z;
  const auto H5_t = (G6 - G5) / Z;
  const auto H6_t = (G7 - G6) / Z;
  const auto H7_t = (G8 - G7) / Z8;

  return { H1_t, H2_t, H3_t, H4_t, H5_t, H6_t, H7_t };
}

std::vector<double> getInitialValues(
  double time,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  constexpr auto H = 0.2;

  const auto H1 = states[0];
  const auto H2 = states[1];
  const auto H3 = states[2];
  const auto H4 = states[3];
  const auto H5 = states[4];
  const auto H6 = states[5];
  const auto H7 = states[6];

  const auto H1_0 = time < 0.1 ? H : H1;
  const auto H2_0 = time < 0.1 ? H : H2;
  const auto H3_0 = time < 0.1 ? H : H3;
  const auto H4_0 = time < 0.1 ? H : H4;
  const auto H5_0 = time < 0.1 ? H : H5;
  const auto H6_0 = time < 0.1 ? H : H6;
  const auto H7_0 = time < 0.1 ? H : H7;

  return { H1_0, H2_0, H3_0, H4_0, H5_0, H6_0, H7_0 };
}

std::vector<double> fittingFunction(
  const double /*time*/,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  const auto H1 = states[0];
  const auto H2 = states[1];
  const auto H3 = states[2];
  const auto H4 = states[3];
  const auto H5 = states[4];
  const auto H6 = states[5];
  const auto H7 = states[6];

  return { H1, H2, H3, H4, H5, H6, H7 };
}

std::vector<double> constraints(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const std::vector<double>& /*modelParameters*/)
{
  return {};
}

}// namespace test_model
