#include "PHARMA.hpp"

#include <cmath>

namespace test_model
{

std::vector<double> ode(
  const double time,
  const std::vector<double>& states,
  const std::vector<double>& modelParameters)
{
  constexpr auto dose = 48.15;

  const auto k1 = modelParameters[0];
  const auto k2 = modelParameters[1];
  const auto kd = modelParameters[2];
  const auto km = modelParameters[3];
  const auto kc = modelParameters[4];
  const auto vb = modelParameters[5];
  const auto vs = modelParameters[6];
  const auto vm = modelParameters[7];
  const auto et = modelParameters[8];
  const auto d = states[0];
  const auto s = states[1];
  const auto m = states[2];

  const auto i = time < et ? dose / et : 0;

  const auto d_t = (i - (k1 + kd + kc) * d + k2 * s) / vb;
  const auto s_t = (k1 * d - k2 * s) / vs;
  const auto m_t = (kc * d - km * m) / vm;
  const auto a_t = kd * d;
  const auto b_t = km * m;

  return { d_t, s_t, m_t, a_t, b_t };
}

std::vector<double> getInitialValues(
  double time,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  const auto d = states[0];
  const auto s = states[1];
  const auto m = states[2];
  const auto a = states[3];
  const auto b = states[4];

  constexpr auto tol = 0.00001;

  const auto d_0 = time < tol ? 0.0 : d;
  const auto s_0 = time < tol ? 0.0 : s;
  const auto m_0 = time < tol ? 0.0 : m;
  const auto a_0 = time < tol ? 0.0 : a;
  const auto b_0 = time < tol ? 0.0 : b;// Bug in PCOMP model?!

  return { d_0, s_0, m_0, a_0, b_0 };
}

std::vector<double> fittingFunction(
  const double /*time*/,
  const std::vector<double>& states,
  const std::vector<double>& /*modelParameters*/)
{
  const auto d = states[0];
  const auto m = states[2];
  const auto a = states[3];
  const auto b = states[4];

  return { d, m, a, b };
}

std::vector<double> constraints(
  double /*time*/,
  const std::vector<double>& /*states*/,
  const std::vector<double>& /*modelParameters*/)
{
  return {};
}

}// namespace test_model
