#include "testUtility.hpp"

#include <algorithm>
#include <functional>
#include <iterator>
#include <random>

namespace
{

std::function<double()> create_double_generator_with_range(
  const double lower,
  const double upper,
  const std::uint_fast32_t seed)
{
  std::mt19937 mersenne_engine(seed);
  const auto generator = [lower, upper, mersenne_engine]() mutable {
    std::uniform_real_distribution<> dist(lower, upper);
    return dist(mersenne_engine);
  };

  return generator;
}

}// namespace

namespace testutility
{

std::vector<double> create_random_vector_of_doubles(
  const std::size_t numberOfElements,
  const double lowerBound,
  const double upperBound,
  const std::uint_fast32_t seed)
{
  const auto generator = create_double_generator_with_range(lowerBound, upperBound, seed);

  std::vector<double> randomVector;
  randomVector.reserve(numberOfElements);

  std::generate_n(std::back_inserter(randomVector), numberOfElements, generator);

  return randomVector;
}

double create_random_double(const double lower, const double upper, const std::uint_fast32_t seed)
{
  const auto generator = create_double_generator_with_range(lower, upper, seed);
  return generator();
}

}// namespace testutility
